{********************************************************************}
{   Program:  LEX/YACC-Compiler for PL/0                             }
{   Module :  PL0ERR - Error Message Routines                        }
{   File   :  PL0ERR.PAS                                             }
{   Author :  H. Weber                                               }
{********************************************************************}

UNIT pl0err;

INTERFACE

TYPE
  yesno = (yes, no);
  message = String[80];

VAR
  errorsw : yesno;                {yes: errors have already occured        }
  msg : message;                  {buffer for error messages               }

PROCEDURE errortxt(n : Integer);
PROCEDURE error(n : Integer);

IMPLEMENTATION

  PROCEDURE errortxt(n : Integer);

    {--------------------------------------------------------------------}
    {   The procedure Errortxt returns the error message text correspon- }
    {   ding to a given error number in the global buffer msg.           }
    {--------------------------------------------------------------------}

  BEGIN                           {Errortext}
    CASE n OF
      1 : msg := 'Use "=" instead of ":=" !                                   ';
      2 : msg := 'A number must follow after "=" !                            ';
      3 : msg := 'The identifier must be followed by "=" !                    ';
      4 : msg := 'After "const", "var" and "procedure" follows an identifier! ';
      5 : msg := 'Semicolon or comma missing !                                ';
      7 : msg := 'Statement or declaration expected !                         ';
      8 : msg := 'Symbol following the statements of the block not correct !  ';
      9 : msg := 'Period expected !                                           ';
      10 : msg := 'Incorrect use of a symbol within a statement !              ';
      11 : msg := 'Undeclared identifier !                                     ';
      12 : msg := 'Assignments to a constant or procedure name not allowed !   ';
      13 : msg := 'Assignment operator is ":=" !                               ';
      14 : msg := 'After "call" a procedure identifier is expected !           ';
      15 : msg := 'Constant or variable identifier after call not allowed !    ';
      16 : msg := '"then" expected !                                           ';
      17 : msg := '"end" or ";" expected !                                     ';
      18 : msg := '"do" expected !                                             ';
      19 : msg := 'Illegal symbol following a statement !                      ';
      20 : msg := 'Comparison operator expected !                              ';
      21 : msg := 'procedure identifier within an expression not allowed !     ';
      22 : msg := 'Missing ")" !                                               ';
      23 : msg := 'This symbol cannot follow a factor !                        ';
      24 : msg := 'An Expression may not start with this symbol !              ';
      30 : msg := 'number too large !                                          ';
      31 : msg := 'program too long !                                          ';
      32 : msg := 'procedure or statement nesting too deep !                                ';
      33 : msg := 'identifier expected !                                       ';
      {Erweiterung 18.8.1989 }
      34 : msg := 'semicolon missing !                                         ';
      35 : msg := '"until" expected !                                          ';
      {Erweiterung 20.8.1989 }
      36 : msg := '"of" expected !                                             ';
      37 : msg := 'Case table overflow !                                       ';
      38 : msg := '":" expected !                                              ';
      39 : msg := 'Number expected !                                           ';
      40 : msg := 'Multiple definition of case label !                         ';
      41 : msg := '"end" expected !                                            ';
      {Erweiterung 21.8.1989 }
      42 : msg := 'String expected!                                            ';
      43 : msg := 'Missing "("                                                 ';
      {Erweiterung 18.8.1989 }
      44 : msg := '"to" or "downto" expected!                                  ';
      45 : msg := 'Multiple declaration in block !                             ';
      46 : msg := 'Illegal symbol following a "[" !                            ';
      47 : msg := '"]" expected !                                              ';
      48 : msg := '"[" expected !                                              ';
      49 : msg := 'Array-Variable expected !                                   ';
      50 : msg := 'No array allowed !                                          ';
      {Erweiterung  6.1.1992 }
    END                           {CASE}

  END;                            {Errortext}


  PROCEDURE error(n : Integer);
    {--------------------------------------------------------------------}
    {   The procedure Error marks the Error position and prints a diag-  }
    {   nostic message into the listing file and into the workfile.      }
    {--------------------------------------------------------------------}

  BEGIN {Error} ;
    errorsw := yes;
    errortxt(n);
    WriteLn('Error ', n:2, '  ** ', msg);
  END {Error} ;

BEGIN
  errorsw := no;
END.
