{********************************************************************}
{   Program:  LEX/YACC-Compiler for PL/0                             }
{   Module :  PL0INIT - Initialization Routine                       }
{   File   :  PL0INIT.PAS                                            }
{   Author :  H. Weber                                               }
{********************************************************************}

UNIT pl0init;

INTERFACE
USES pl0code;

CONST
  imax = 32767;                   {maximum integer value                        }
  cmax = 30;                      {maximum number of characters for identifiers }
  nestmax = 3;                    {maximum depth of block nesting               }
  casemax = 10;                   {maximum length of case tables, Erw. 21.8.1989}
  maxstack = 20;                  {maximum length of compiler stacks            }

TYPE
  alfa = String[cmax];

VAR
  { Stacks fr Fixups und div. Block-Gren }
  arr, tx, fixup,
  ppos, dindex : ARRAY[1..maxstack] OF Integer;
  caseinfo : ARRAY[1..maxstack] OF
    RECORD
      i, j, l : Integer;
      casetab : ARRAY[1..casemax] OF
        RECORD
          value, cdindex : Integer;
        END;
      exittab : ARRAY[1..casemax] OF Integer;
    END;

  arrptr, caseptr, txptr, fixupptr,
  pposptr, dindexptr : Integer;
  { Stackpointer dafr }

  id : alfa;                      {last identifier read by scanner   }
  Str : alfa;                     {last string read by scanner       }
  num : Integer;                  {last number read by the scanner   }
  arrsize : Integer;              {size of last array                }
  sourcefile : String[14];        {name of the source input file           }
  listing : Text;                 {listing output file                     }

PROCEDURE initialize;

IMPLEMENTATION

  PROCEDURE initialize;
    {--------------------------------------------------------------------}
    {   The procedure Initialize opens the files, initializes the        }
    {   table mnemo and various other pointers and switches.             }
    {--------------------------------------------------------------------}

  BEGIN                           {Initialize}
    mnemo[add] := ' ADD '; mnemo[sub] := ' SUB ';
    mnemo[mpy] := ' MPY '; mnemo[dvd] := ' DVD ';
    mnemo[neg] := ' NEG '; mnemo[od] := ' OD  ';
    mnemo[eq] := ' EQ  '; mnemo[ne] := ' NE  ';
    mnemo[ls] := ' LS  '; mnemo[gt] := ' GT  ';
    mnemo[le] := ' LE  '; mnemo[ge] := ' GE  ';
    mnemo[lit] := ' LIT '; mnemo[ret] := ' RET ';
    mnemo[cal] := ' CAL '; mnemo[lod] := ' LOD ';
    mnemo[sto] := ' STO '; mnemo[Int] := ' INT ';
    mnemo[jpu] := ' JPU '; mnemo[jpc] := ' JPC ';
    mnemo[inp] := ' INP '; mnemo[prn] := ' PRN '; { Erw. 18.8.1989 }
    mnemo[prs] := ' PRS '; mnemo[prl] := ' PRL '; { Erw. 18.8.1989 }
    mnemo[jpx] := ' JPX '; mnemo[ldl] := ' LDL '; { Erw. 21.8.1989 }
    mnemo[lda] := ' LDA '; mnemo[feu] := ' FEU '; { Erw.  2.1.1990 }
    mnemo[fed] := ' FED '; mnemo[fru] := ' FRU '; { Erw.  2.1.1990 }
    mnemo[frd] := ' FRD '; mnemo[asc] := ' ASC '; { Erw.  2.1.1990 }
    mnemo[sta] := ' STA '; mnemo[drf] := ' DRF '; { Erw.  8.1.1992 }
  END;                            { Initialize }

BEGIN
  WriteLn('LEX/YACC-Compiler for PL/0 by H.Weber (1991)');
  WriteLn;
  initialize;
END.
